/****************************************************************************
** Meta object code from reading C++ file 'displaymanageradaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "displaymanageradaptor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'displaymanageradaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DisplayManagerAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto DisplayManagerAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN21DisplayManagerAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DisplayManagerAdaptor",
        "D-Bus Interface",
        "org.freedesktop.DisplayManager",
        "D-Bus Introspection",
        "  <interface name=\"org.freedesktop.DisplayManager\">\n    <signal"
        " name=\"SeatAdded\">\n      <arg type=\"o\" name=\"seat\"/>\n    <"
        "/signal>\n    <signal name=\"SeatRemoved\">\n      <arg type=\"o\""
        " name=\"seat\"/>\n    </signal>\n    <signal name=\"SessionAdded\""
        ">\n      <arg type=\"o\" name=\"session\"/>\n    </signal>\n    <s"
        "ignal name=\"SessionRemoved\">\n      <arg type=\"o\" name=\"sessi"
        "on\"/>\n    </signal>\n    <property access=\"read\" type=\"ao\" n"
        "ame=\"Seats\"/>\n    <property access=\"read\" type=\"ao\" name=\""
        "Sessions\"/>\n  </interface>\n",
        "SeatAdded",
        "",
        "QDBusObjectPath",
        "seat",
        "SeatRemoved",
        "SessionAdded",
        "session",
        "SessionRemoved",
        "Seats",
        "QList<QDBusObjectPath>",
        "Sessions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SeatAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'SeatRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'SessionAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 11 },
        }}),
        // Signal 'SessionRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Seats'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Sessions'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(15, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DisplayManagerAdaptor, qt_meta_tag_ZN21DisplayManagerAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DisplayManagerAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DisplayManagerAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DisplayManagerAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DisplayManagerAdaptorE_t>.metaTypes,
    nullptr
} };

void DisplayManagerAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayManagerAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SeatAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 1: _t->SeatRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 2: _t->SessionAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        case 3: _t->SessionRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DisplayManagerAdaptor::*)(const QDBusObjectPath & )>(_a, &DisplayManagerAdaptor::SeatAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManagerAdaptor::*)(const QDBusObjectPath & )>(_a, &DisplayManagerAdaptor::SeatRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManagerAdaptor::*)(const QDBusObjectPath & )>(_a, &DisplayManagerAdaptor::SessionAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayManagerAdaptor::*)(const QDBusObjectPath & )>(_a, &DisplayManagerAdaptor::SessionRemoved, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->seats(); break;
        case 1: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->sessions(); break;
        default: break;
        }
    }
}

const QMetaObject *DisplayManagerAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DisplayManagerAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DisplayManagerAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int DisplayManagerAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DisplayManagerAdaptor::SeatAdded(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DisplayManagerAdaptor::SeatRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DisplayManagerAdaptor::SessionAdded(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DisplayManagerAdaptor::SessionRemoved(const QDBusObjectPath & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
